"""
All settings (constants) for the whole library are stored in this file.
These settings are often used in multiple files, thus why it is all in one file.
"""
### LMS adaptive filter

MU_LMS = 0.01
""" Default learning rate for LMS algorithm """

MU_LMS_MIN = 0.
""" Minimal allowed learning rate for LMS algorithm """

MU_LMS_MAX = 1000.
""" Maximal allowed learning rate for LMS algorithm """


### NLMS adaptive filter

MU_NLMS = 0.1
""" Default learning rate for NLMS algorithm """

MU_NLMS_MIN = 0.
""" Minimal allowed learning rate for NLMS algorithm """

MU_NLMS_MAX = 1000.
""" Minimal allowed learning rate for NLMS algorithm """

EPS_NLMS = 1.
""" Default regularization term for NLMS algorithm """

EPS_NLMS_MIN = 0.
""" Minimal allowed regularization term for NLMS algorithm """

EPS_NLMS_MAX = 1000.
""" Minimal allowed regularization term for NLMS algorithm """

### OCNLMS adaptive filter

MU_OCNLMS = 0.1
""" Default learning rate for OC-NLMS algorithm """

MU_OCNLMS_MIN = 0.
""" Minimal allowed learning rate for OC-NLMS algorithm """

MU_OCNLMS_MAX = 1000.
""" Minimal allowed learning rate for OC-NLMS algorithm """

EPS_OCNLMS = 1.
""" Default regularization term for OC-NLMS algorithm """

EPS_OCNLMS_MIN = 0.
""" Minimal allowed regularization term for OC-NLMS algorithm """

EPS_OCNLMS_MAX = 1000.
""" Minimal allowed regularization term for OC-NLMS algorithm """

MEM_OCNLMS = 100
""" Default memory size for OC-NLMS algorithm. """

##### RLS adaptive filter

MU_RLS = 0.99
""" Default learning rate for NLMS algorithm """

MU_RLS_MIN = 0.
""" Minimal allowed learning rate for NLMS algorithm """

MU_RLS_MAX = 1.
""" Minimal allowed learning rate for NLMS algorithm """

EPS_RLS = 0.1
""" Default regularization term for RLS algorithm """

EPS_RLS_MIN = 0.
""" Minimal allowed regularization term for RLS algorithm """

EPS_RLS_MAX = 1.
""" Minimal allowed regularization term for RLS algorithm """


##### GNGD adaptive filter

MU_GNGD = 1.
""" Default learning rate for NLMS algorithm """

MU_GNGD_MIN = 0.
""" Minimal allowed learning rate for NLMS algorithm """

MU_GNGD_MAX = 1000.
""" Minimal allowed learning rate for NLMS algorithm """


EPS_GNGD = 1.
""" Default regularization term for RLS algorithm """

EPS_GNGD_MIN = 0.
""" Minimal allowed regularization term for GNGD algorithm """

EPS_GNGD_MAX = 10.
""" Minimal allowed regularization term for GNGD algorithm """


RO_GNGD = 0.1
""" Default regularization term for GNGD algorithm """

RO_GNGD_MIN = 0.
""" Minimal allowed regularization term for GNGD algorithm """

RO_GNGD_MAX = 1.
""" Minimal allowed regularization term for GNGD algorithm """






"""
This sub-module contains functions usefull for data preprocessing.
"""
from __future__ import division
import numpy as np

### data normalization
def standardize(x, offset=None, scale=None):
    """
    This function standardizes the series.

    xs = (x - offset) / scale

    Args:

    * `x` : series (1 dimensional array)

    Kwargs:

    * `offset` : offset to remove (float). If not given,
        the mean value of `x` is used.

    * `scale` : scale (float). If not given,
        the standard deviation of `x` is used.
        
    Returns:

    * `xs` : standardized series

    Example:


        >>> import numpy as np
        >>> import padasip as pa
        >>> x = np.random.random(1000)
        >>> x.mean()
        0.49755420774866677
        >>> x.std()
        0.29015765297767376
        >>> xs = pa.standardize(x)
        >>> xs.mean()
        1.4123424652012772e-16
        >>> xs.std()
        0.99999999999999989

    """
    if offset == None:
        offset = np.array(x).mean()
    else:
        try:
            offset = float(offset)
        except:
            raise ValueError('The argument offset is not None or float') 
    if scale == None:
        scale = np.array(x).std()
    else:
        try:
            scale = float(scale)
        except:
            raise ValueError('The argument scale is not None or float')    
    try:
        x = np.array(x, dtype="float64")
    except:
        raise ValueError('The argument x is not numpy array or similar.')         
    return (x - offset) / scale
    

def standardize_back(xs, offset, scale):
    """
    This function transforms series to the original score.

    x = xs * scale + offset

    Args:

    * `xs` : standardized input (1 dimensional array)

    * `offset` : offset to add (float).

    * `scale` : scale (float).
        
    Returns:

    * `x` : original (destandardised) series
    """
    try:
        offset = float(offset)
    except:
        raise ValueError('The argument offset is not None or float.') 
    try:
        scale = float(scale)
    except:
        raise ValueError('The argument scale is not None or float.')
    try:
        xs = np.array(xs, dtype="float64")
    except:
        raise ValueError('The argument xs is not numpy array or similar.')
    return xs*scale + offset


### INPUT MATRIX constructors

def input_from_history(a, n, bias=False):
    """
    This function creates input matrix from historical values.

    Args:

    * `a` : series (1 dimensional array)

    * `n` : size of input matrix row (int).
        It means how many samples of previous history you want to use
        as the filter input. It also represents the filter length.

    Kwargs:

    * `bias` : decides if the bias is used (Boolean). If True,
        array of all ones is appended as a last column to matrix `x`.
        So matrix `x` has `n`+1 columns.

    Returns:

    * `x` : input matrix (2 dimensional array)
        constructed from an array `a`. The length of `x`
        is calculated as length of `a` - `n` + 1. 
        If the `bias` is used, then the amount of columns is `n` if not then
        amount of columns is `n`+1).

    Example:

        >>> import numpy as np
        >>> import padasip as pa
        >>> a = np.arange(1, 7, 1)
        >>> a
        array([1, 2, 3, 4, 5, 6])
        >>> pa.input_from_history(a,3)
        array([[1, 2, 3],
               [2, 3, 4],
               [3, 4, 5],
               [4, 5, 6]])

    """
    if not type(n) == int:
        raise ValueError('The argument n must be int.')
    if not n > 0:
        raise ValueError('The argument n must be greater than 0')
    try:
        a = np.array(a, dtype="float64")
    except:
        raise ValueError('The argument a is not numpy array or similar.')
    x = np.array([a[i:i+n] for i in range(len(a)-n+1)]) 
    if bias:
        x = np.vstack((x.T, np.ones(len(x)))).T
    return x



"""
Current version: |version| (:ref:`changelog-label`)

This library is designed to simplify adaptive signal 
processing tasks within python
(filtering, prediction, reconstruction, classification).
For code optimisation,
this library uses `Numpy <http://www.numpy.org/>`_ for array operations.

Also in this library is presented some new methods
for adaptive signal processing.
The library is designed to be used with datasets and also with 
real-time measuring (sample-after-sample feeding).

.. toctree::
    :maxdepth: 2
    
    index


License
************

This project is under `MIT License <https://en.wikipedia.org/wiki/MIT_License>`_.

Instalation
************
With `pip <https://pypi.python.org/pypi/pip>`_ from terminal: ``$ pip install padasip``

Or download you can download the source codes from Github
(`link <https://github.com/matousc89/padasip>`_)


Tutorials
***********

All tutorials are created as Jupyter notebooks.
You can open the tutorial as html, or you can download it as notebook.

* Noise Cancelation, Identification and Prediction with Adaptive Filters
    (`see online <tutorials/tutorial1.html>`_) (`download notebook <ipynb/tutorial1.ipynb>`_)

* Adaptive Novelty Detection
    (`see online <tutorials/tutorial2.html>`_) (`download notebook <ipynb/tutorial2.ipynb>`_)
    
* Adaptive filters in Real-time with PADASIP Module
    (`see online <tutorials/tutorial3.html>`_) (`download notebook <ipynb/tutorial3.ipynb>`_)

* Multi-layer Perceptron (MLP) Neural Network - Basic Examples
    (`see online <tutorials/mlp_tutorial.html>`_) (`download notebook <ipynb/mlp_tutorial.ipynb>`_)


The User Quide
***************

If you need to know something what is not covered by tutorials,
check the complete documentation here



.. toctree::
    :maxdepth: 2
    
    sources/preprocess
    sources/filters
    sources/ann


Contact
**********

By email: matousc@gmail.com


Indices and tables
*******************
* :ref:`genindex`
* :ref:`modindex`
* :ref:`search`

"""
from padasip.preprocess import *
from padasip.filters.shortcuts import *
import padasip.ann
import padasip.filters



from distutils.core import setup

def readme():
    try:
        with open('README.rst') as f:
            return f.read()
    except:
        pass

setup(
    name = 'padasip',
    packages = ['padasip'],
    version = '0.3',
    description = 'Python Adaptive Signal Processing',
    long_description = readme(),
    author = 'Matous Cejnek',
    author_email = 'matousc@gmail.com',
    license = 'MIT',
    url = 'https://github.com/matousc89/padasip.git',
    download_url = 'https://github.com/matousc89/padasip.git/tarball/0.1',
    keywords = ['adaptive', 'signal-processing'],
    install_requires=[
        'numpy',
    ],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Topic :: Adaptive Technologies',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Software Development',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Programming Language :: Python',
    ],
)
